package com.qmth.eds.common.tools;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.qmth.eds.common.contant.SysSettingConstant;
import com.qmth.eds.common.entity.ExamSyncStudentTemp;
import com.qmth.eds.common.enums.ExceptionResultEnum;
import com.qmth.eds.common.util.EhcacheUtil;
import com.qmth.eds.common.util.HttpKit;
import com.qmth.eds.service.CommonService;
import com.qmth.eds.service.impl.WhuDataSyncServiceImpl;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.time.temporal.ChronoUnit;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.atomic.AtomicInteger;

/**
 * 武汉大学工具类
 */
@Component
public class CdutUtils {
    private static final Logger log = LoggerFactory.getLogger(WhuDataSyncServiceImpl.class);

    private final static int CODE_SUCCESS = 10000;

    @Resource
    EhcacheUtil ehcacheUtil;

    @Resource
    CommonService commonService;

    /**
     * 获取token
     */
    public String getAccessToken(Long schoolId) {
        String key = "accessToken" + schoolId;
        Object o = ehcacheUtil.get(key);
        if (o != null) {
            return String.valueOf(o);
        }

        String appKey = commonService.getSysSettingValue(schoolId, SysSettingConstant.CDUT_APP_KEY);
        String appSecret = commonService.getSysSettingValue(schoolId, SysSettingConstant.CDUT_APP_SECRET);
        String tokenUrl = commonService.getSysSettingValue(schoolId, SysSettingConstant.CDUT_TOKEN_URL);
        validUrl(appKey, appSecret, tokenUrl);
        try {
            //参数
            Map<String, String> paramMap = new HashMap<>();
            paramMap.put("appKey", appKey);
            paramMap.put("appSecret", appSecret);

            String result = HttpKit.sendPost(tokenUrl, paramMap, null);
            JSONObject jsonResult = JSON.parseObject(result);
            log.info("参数：{}", JSONObject.toJSONString(paramMap));
            log.info("结果：{}", result);
            if (jsonResult.containsKey("code") && CODE_SUCCESS == jsonResult.getIntValue("code")) {
                String resultString = jsonResult.getString("result");
                JSONObject jsonToken = JSON.parseObject(resultString);
                if (jsonToken.containsKey("access_token")) {
                    String accessToken = jsonToken.getString("access_token");
                    int expiresIn = jsonToken.getIntValue("expires_in");
                    // 缓存
                    ehcacheUtil.set(key, accessToken, expiresIn, ChronoUnit.SECONDS);
                    return accessToken;
                }
            }
            return null;
        } catch (Exception e) {
            throw ExceptionResultEnum.ERROR.exception(e.getMessage());
        }
    }

    /**
     * 获取考务数据
     */
    public List<ExamSyncStudentTemp> getKwData(Long schoolId, String token) {
        String kwUrl = commonService.getSysSettingValue(schoolId, SysSettingConstant.CDUT_KW_URL);
        validUrl(kwUrl);
        try {
            // 循环查
            AtomicInteger offset = new AtomicInteger(1);
            int perPage = 1000;
            int totalCount = 0;
            List<ExamSyncStudentTemp> subExamSyncStudentTemps = null;
            List<ExamSyncStudentTemp> examSyncStudentTemps = new ArrayList<>();
            do {
                Map<String, String> paramMap = new HashMap<>();
                paramMap.put("access_token", token);
                paramMap.put("page", String.valueOf(offset.getAndIncrement()));
                paramMap.put("per_page", String.valueOf(perPage));

                String data = HttpKit.sendGet(kwUrl, paramMap, null);
                JSONObject dataResult = JSON.parseObject(data);
                if (dataResult.containsKey("code") && CODE_SUCCESS == dataResult.getIntValue("code")) {
                    String resultString = dataResult.getString("result");
                    JSONObject jsonResult = JSON.parseObject(resultString);
                    if (totalCount == 0) {
                        totalCount = jsonResult.getIntValue("total");
                    }
                    if (jsonResult.containsKey("data")) {
                        String dataString = jsonResult.getString("data");
                        subExamSyncStudentTemps = JSON.parseArray(dataString, ExamSyncStudentTemp.class);
                        if (!subExamSyncStudentTemps.isEmpty()) {
                            examSyncStudentTemps.addAll(subExamSyncStudentTemps);
                        }
                    }
                }
            } while (!subExamSyncStudentTemps.isEmpty());

            if (totalCount != examSyncStudentTemps.size()) {
                throw ExceptionResultEnum.ERROR.exception(String.format("不一致，接口返回总数%d，数据解析总数%d", totalCount, examSyncStudentTemps.size()));
            }

            return examSyncStudentTemps;
        } catch (Exception e) {
            throw ExceptionResultEnum.ERROR.exception(e.getMessage());
        }
    }

    /**
     * 校验url是否配置
     *
     * @param urls URL数组
     */
    private void validUrl(String... urls) {
        if (StringUtils.isAnyBlank(urls)) {
            throw ExceptionResultEnum.ERROR.exception("接口参数未配置");
        }
    }

    private List<ExamSyncStudentTemp> listTemps() {
        List<ExamSyncStudentTemp> examSyncStudentTemps = new ArrayList<>();
        for (int i = 0; i < 4; i++) {
            ExamSyncStudentTemp examSyncStudentTemp = new ExamSyncStudentTemp();
            examSyncStudentTemp.setXnm("100" + i);
            examSyncStudentTemp.setXqm("200" + i);
            examSyncStudentTemp.setJxbId("300" + i);
            examSyncStudentTemp.setJxbmc("教学班" + i);
            examSyncStudentTemp.setKch("400" + i);
            examSyncStudentTemp.setKcmc("课程" + i);
            examSyncStudentTemp.setJgh("500" + i);
            examSyncStudentTemp.setXm("姓名" + i);
            examSyncStudentTemp.setKkbm("开课学院" + i);
            examSyncStudentTemp.setXf("35");
            examSyncStudentTemp.setXh("100000100" + i);
            examSyncStudentTemp.setXsxm("学生" + i);
            examSyncStudentTemp.setJgmc("机构" + i);
            examSyncStudentTemp.setZymc("专业" + i);
            examSyncStudentTemp.setCxbj("0");
            examSyncStudentTemp.setNjdmId("年级" + i);
            examSyncStudentTemp.setZwh("1" + i);
            examSyncStudentTemp.setCdmc("考场" + i);
            examSyncStudentTemp.setKsbz("无");
            examSyncStudentTemps.add(examSyncStudentTemp);
        }
        return examSyncStudentTemps;
    }
}
