package com.qmth.eds.common.util;

import com.qmth.eds.common.entity.TBSession;
import org.ehcache.Cache;
import org.ehcache.CacheManager;
import org.ehcache.config.CacheConfiguration;
import org.ehcache.config.builders.CacheConfigurationBuilder;
import org.ehcache.config.builders.CacheManagerBuilder;
import org.ehcache.config.builders.ExpiryPolicyBuilder;
import org.ehcache.config.builders.ResourcePoolsBuilder;
import org.ehcache.config.units.EntryUnit;
import org.ehcache.config.units.MemoryUnit;
import org.springframework.stereotype.Component;

import java.time.temporal.ChronoUnit;
import java.time.temporal.TemporalUnit;

/**
 * ehcache工具类
 */
@Component
public class EhcacheUtil {

    private static String DEFAULT_CACHE = "dynamicCache";

    private CacheManager cacheManager;

    /**
     * 设置值
     *
     * @param key      key
     * @param value    值
     * @param time     过期时间
     * @param timeUnit 时间类型
     */
    public void set(String key, String value, long time, ChronoUnit timeUnit) {
        set(DEFAULT_CACHE, key, value, time, timeUnit);
    }

    /**
     * 查询
     *
     * @param key
     * @return String
     */
    public String get(String key) {
        return get(DEFAULT_CACHE, key);
    }

    /**
     * 设置值
     *
     * @param cacheName 缓存名
     * @param key       key
     * @param value     值
     * @param time      过期时间
     * @param timeUnit  时间类型
     */
    public void set(String cacheName, String key, String value, long time, ChronoUnit timeUnit) {
        Cache<String, String> cache = getCache(cacheName, time, timeUnit);
        cache.put(key, value);
    }

    /**
     * 设置用户session值
     *
     * @param cacheName 缓存名
     * @param key       key
     * @param value     值
     * @param time      过期时间
     * @param timeUnit  时间类型
     */
    public void setUserSession(String cacheName, String key, TBSession value, long time, ChronoUnit timeUnit) {
        Cache<String, TBSession> cache = getCache(cacheName, TBSession.class, time, timeUnit);
        cache.put(key, value);
    }

    /**
     * 查询
     *
     * @param cacheName 缓存名
     * @param key       key
     * @return String
     */
    public String get(String cacheName, String key) {
        Cache<String, String> cache = getCache(cacheName, 0, null);
        return cache.get(key);
    }

    /**
     * 查询用户session值
     *
     * @param cacheName 缓存名
     * @param key       key
     * @return String
     */
    public TBSession getUserSession(String cacheName, String key) {
        Cache<String, TBSession> cache = getCache(cacheName, TBSession.class, 0, null);
        return cache.get(key);
    }

    /**
     * 移除
     *
     * @param cacheName 缓存名
     * @param key       key
     */
    public void remove(String cacheName, String key) {
        Cache<String, String> cache = getCache(cacheName, 0, null);
        cache.remove(key);
    }

    /**
     * 移除用户session值
     *
     * @param cacheName 缓存名
     * @param key       key
     */
    public void removeUserSession(String cacheName, String key) {
        Cache<String, TBSession> cache = getCache(cacheName, TBSession.class, 0, null);
        cache.remove(key);
    }

    /**
     * 移除
     *
     * @param key key
     */
    public void remove(String key) {
        remove(DEFAULT_CACHE, key);
    }

    /**
     * 创建新缓存
     *
     * @param cacheName 缓存名
     * @param time      过期时间
     * @param timeUnit  时间类型
     * @return Cache<String, String>
     */
    public Cache<String, String> getCache(String cacheName, long time, TemporalUnit timeUnit) {
        return getCache(cacheName, String.class, time, timeUnit);
    }

    /**
     * 创建新缓存
     *
     * @param cacheName 缓存名
     * @param time      过期时间
     * @param timeUnit  时间类型
     * @return Cache<String, String>
     */
    public <T> Cache<String, T> getCache(String cacheName, Class<T> c, long time, TemporalUnit timeUnit) {
        if (cacheManager == null) {
            cacheManager = cacheManager();
        }
        Cache<String, T> cache = cacheManager.getCache(cacheName, String.class, c);
        if (cache == null) {
            CacheConfiguration cacheConfiguration = CacheConfigurationBuilder.newCacheConfigurationBuilder(String.class, c, ResourcePoolsBuilder.newResourcePoolsBuilder()
                            .heap(100, EntryUnit.ENTRIES)
                            .offheap(1, MemoryUnit.GB))
                    .withExpiry(ExpiryPolicyBuilder.timeToLiveExpiration(java.time.Duration.of(time, timeUnit)))
                    .build();
            cache = cacheManager.createCache(cacheName, cacheConfiguration);
        }
        return cache;
    }

    /**
     * 创建CacheManager
     *
     * @return CacheManager
     */
    public CacheManager cacheManager() {
        CacheManager cacheManager = CacheManagerBuilder.newCacheManagerBuilder().build();
        cacheManager.init();
        return cacheManager;
    }
}